﻿using System;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;

namespace SuperPutty.Scp
{
    public partial class FileTransferView : UserControl
    {
        private bool initialized = false;

        public FileTransferView()
        {
            InitializeComponent();
        }

        public FileTransferView(IFileTransferPresenter presenter) : this()
        {
            Initialize(presenter);
        }

        public void Initialize(IFileTransferPresenter presenter)
        {
            if (initialized) return;

            this.Presenter = presenter;
            this.bindingSource.DataSource = presenter.ViewModel.FileTransfers;
            initialized = true;
        }

        #region Context Menu

        private void contextMenu_Opening(object sender, CancelEventArgs e)
        {
            Point p = PointToClient(MousePosition);
            DataGridView.HitTestInfo hit = this.grid.HitTest(p.X, p.Y);
            if (hit.Type == DataGridViewHitTestType.Cell)
            {
                // toggle on/off the actions based on view model
                FileTransferViewItem item = (FileTransferViewItem) ((DataGridViewRow) grid.Rows[hit.RowIndex]).DataBoundItem;
                this.runAgainToolStripMenuItem.Enabled = item.CanRestart;
                this.cancelToolStripMenuItem.Enabled = item.CanCancel;
                this.deleteToolStripMenuItem.Enabled = item.CanDelete;
            }
            else
            {
                // Only open on cells
                e.Cancel = true;
            }
        }

        private void grid_CellMouseDown(object sender, DataGridViewCellMouseEventArgs e)
        {
            if (MouseButtons == MouseButtons.Right)
            {
                if (!grid.Rows[e.RowIndex].Selected)
                {
                    grid.ClearSelection();
                    grid.Rows[e.RowIndex].Selected = true;
                }
            }
        }


        private void cancelToolStripMenuItem_Click(object sender, EventArgs e)
        {
            FileTransferViewItem item = GetSelectedItem<FileTransferViewItem>();
            if (item != null)
            {
                this.Presenter.Cancel(item.Id);
            }
        }

        private void deleteToolStripMenuItem_Click(object sender, EventArgs e)
        {
            FileTransferViewItem item = GetSelectedItem<FileTransferViewItem>();
            if (item != null)
            {
                this.Presenter.Remove(item.Id);
            }
        }

        private void runAgainToolStripMenuItem_Click(object sender, EventArgs e)
        {
            FileTransferViewItem item = GetSelectedItem<FileTransferViewItem>();
            if (item != null)
            {
                this.Presenter.Restart(item.Id);
            }
        }

        #endregion

        T GetSelectedItem<T>()
        {
            T item = default(T);
            foreach (DataGridViewRow row in this.grid.SelectedRows)
            {
                item = (T) row.DataBoundItem;
                break;
            }
            return item;
        }

        IFileTransferPresenter Presenter { get; set; }
    }
}
